@@ -43,9 +43,17 @@
 
 #include "libxnee/xnee.h"
 #include "libxnee/print.h"
+#include "libxnee/xnee_dl.h"
+#include "libxnee/xnee_sem.h"
+#include "libxnee/xnee_setget.h"
+#include "libxnee/datastrings.h"
 
 
 
+/* internal prototypes */
+int
+get_modifier_sub(xnee_data *xd,  char *mod_str);
+
 
 
 
@@ -87,9 +95,9 @@ xnee_setup_display (xnee_data *xd)
   xnee_verbose((xd, "display data    %d\n" , (int) xd->data));
   xnee_verbose((xd, "display control %d\n" , (int) xd->control));
   xnee_verbose((xd, "display fake    %d\n" , (int) xd->fake));
-  /* return 0 on success, 1  indicats error opening the displays */
+  /* return XNEE_OK on success, 1  indicats error opening the displays */
   if ( ( xd->data != NULL ) && (xd->control) && (xd->fake) )
-    return 0;
+    return XNEE_OK;
   else 
     return 1;
 }
@@ -223,6 +231,24 @@ xnee_open_display(xnee_data* xd)
 }
 
 
+
+int  
+xnee_free_file (xnee_data *xd, char *file_name, FILE* file)
+{
+  
+  xnee_verbose((xd, "Closing file=%s fd=%d\n", file_name, file));
+  if ( file_name != NULL) 
+    {
+      free (file_name);
+    }
+  if ( file != 0) 
+    {
+      fclose (file);
+    }
+  return XNEE_OK;
+}
+
+
 /**************************************************************
  *                                                            *
  * xnee_close_down                                            *
@@ -240,7 +266,7 @@ xnee_close_down(xnee_data* xd)
   if (xd->plugin_handle!=NULL)
     {
       xnee_verbose((xd, "Closing plugin lib "));
-      xnee_dlclose(xd->plugin_handle);
+      xnee_dlclose(xd, xd->plugin_handle);
     }
 
   if (xd->buf_sem!=0)
@@ -287,16 +313,12 @@ xnee_close_down(xnee_data* xd)
   XNEE_DEBUG ( (stderr ," --> xnee_close_down() at 0.5 \n"  ));
   xnee_verbose((xd, "closing fds\n"));
   
-  if ( xd->out_name != NULL) 
-    {
-      xnee_verbose((xd, "Closing output file=%s fd=%d\n", xd->out_name, xd->out));
-      fclose (xd->out);
-    }
-  if ( xd->err_name != NULL) 
-    {
-      xnee_verbose((xd, "Closing error file=%s fd=%d\n", xd->err_name, xd->err));
-      fclose (xd->err);
-    }
+
+  xnee_free_file (xd, xd->out_name,  xd->out_file);
+  xnee_free_file (xd, xd->err_name,  xd->err_file);
+  xnee_free_file (xd, xd->data_name, xd->data_file);
+  xnee_free_file (xd, xd->rc_name,   xd->rc_file);
+
   XNEE_DEBUG ( (stderr ," --> xnee_close_down() at 0.6 \n"  ));
   xnee_verbose((xd, "Freeeing data "));
   xnee_free_xnee_data(xd);
@@ -340,36 +362,46 @@ xnee_init(xnee_data* xd, char *name)
       return XNEE_MEMORY_FAULT;
     }
   /*  xd->human_print = False  ; */
-  xd->plugin_handle = NULL;
-  xd->rec_callback  = xnee_record_dispatch ;
-  xd->rep_callback  = xnee_replay_dispatch ;
+  xd->plugin_handle    = NULL;
+  xd->rec_callback     = xnee_record_dispatch ;
+  xd->rep_callback     = xnee_replay_dispatch ;
+  xd->buffer_verbose_fp= fprintf ;
+  xd->verbose_fp       = vfprintf ;
+  xd->data_fp          = fprintf ;
   /* TODO   xd->sync_fun     = xnee_replay_dispatch ; */
 
   xd->verbose       = False  ; 
   xd->buf_verbose   = False  ; 
   xd->all_clients   = True   ; 
   xd->sync          = True   ; 
-  xd->recorder      = True   ; 
-  xd->out           = stdout ;
-  xd->err           = stderr ;
-  xd->sf            = NULL   ;
+  xd->mode          = XNEE_RECORDER   ; 
+
+  xd->data_file     = stdin  ;
+  xd->out_file      = stdout ;
+  xd->err_file      = stderr ;
+  xd->rc_file       = NULL   ;
+  xd->buffer_file   = stderr ;
+
+  xd->data_name     = NULL  ;
+  xd->out_name      = NULL  ;
+  xd->err_name      = NULL  ;
+  xd->rc_name       = NULL  ;
+
   xd->display       = getenv ("DISPLAY");
-  xd->out           = stdout ; 
-  xd->err           = stderr ;
   xd->distr_list    = NULL   ;
   xd->distr_list_size = 0   ;
   xd->cont          = True ;
-  xd->stop_mod      = 0 ;
-  xd->stop_key      = 0 ;
   xd->force_replay  = False;
 
   xd->buf_sem = (sem_t *) malloc (sizeof(sem_t));
   xnee_sem_init (xd, xd->buf_sem, 0, 1);
 
-  xnee_init_names();
+  /*  Not done until needed
+      xnee_init_names();
+  */
 
   xnee_verbose((xd, "<--- xnee_init\n"));
-  return 0;
+  return XNEE_OK;
 }
 
 
@@ -383,18 +415,18 @@ int
 xnee_err_handler(Display* dpy, XErrorEvent* ev)
 {
   int len=100;
-  char tmp[len];
+  char tmp[100];
   
   printf ("xnee - X11 error handler \n");
   printf ("   type       %d \n",  ev->type);
   printf ("   serial     %lu \n", ev->serial);
-  printf ("   error code %lu \n", ev->error_code);
-  printf ("   major code %lu \n", ev->request_code);
-  printf ("   minor code %lu \n", ev->minor_code);
+  printf ("   error code %d \n", ev->error_code);
+  printf ("   major code %d \n", ev->request_code);
+  printf ("   minor code %d \n", ev->minor_code);
   
   XGetErrorText (dpy, ev->error_code, tmp, len);
   printf ("   Message: %s\n", tmp);
-  return 0;
+  return XNEE_OK;
 }
 
 
@@ -441,12 +473,14 @@ get_modifier_from_mapping_sub(Display *display, char *mod_str)
 int
 get_modifier_from_mapping(Display *display, char *mod_str) 
 {
-  XModifierKeymap *mod_keymap;
-  KeyCode *keycode_ptr;
-  KeySym my_keysym;
-  int i, j, max_keys;
+  /*
+    XModifierKeymap *mod_keymap;
+    KeyCode *keycode_ptr;
+    KeySym my_keysym;
+    int i, j, max_keys;
+  */
   int ret;
-
+    
   if (xnee_check(mod_str, "Alt", "Alt"))
     {
       ret = get_modifier_from_mapping_sub(display, "Alt_L");
@@ -550,7 +584,7 @@ is_last (xnee_data *xd,  char *mod_strs)
     {
       if (tmp[i]=='+')
 	{
-	  return 0;
+	  return XNEE_OK;
 	}
     }
   return 1;
@@ -577,18 +611,19 @@ xkm_rem_blanks (char *array)
 	  idx--;
 	}
     }
+  return XNEE_OK;
 }
 
 int
 get_modifier(xnee_data *xd,  char *mod_strs) 
 {
   int ret=0;
-  int try_ret;
+  /*  int try_ret;*/
   char *tmp;
   char mod_head[20]="";
   char mod_tail[40]="";
   int i;
-  int last=0;
+  /*  int last=0;*/
   int len=strlen(mod_strs);
   int found=0;
   static int _level=0;
@@ -701,85 +736,203 @@ xget_modifier(xnee_data *xd, char *mod_str)
 
 /**************************************************************
  *                                                            *
- * xnee_grab_stop_key                                         *
+ * xnee_grab_key                                              *
  *                                                            *
  *                                                            *
  **************************************************************/
 int 
-xnee_grab_stop_key (xnee_data* xd)
+xnee_grab_key (xnee_data* xd, int mode, char *mod_key)
 {
-  int ret=0;
   int window;
   int screen;
+  xnee_km_tuple km;
+
+  xnee_verbose((xd, "----> xnee_grab_key\n"));
+
+  xnee_verbose((xd, "----  xnee_grab_key mod_key=%s\n", mod_key));
+  xnee_get_km_tuple (xd, &km, mod_key);
+  xnee_verbose((xd, "----  xnee_grab_key mod=%d\n", km.modifier));
+  xnee_verbose((xd, "----  xnee_grab_key key=%d\n", km.key));
 
 
-  xnee_verbose((xd, "----> xnee_grab_stop_key\n"));
-  
-  if (xd->stop_key==0)
-      return ret;
+  /* get the key+modifier from xd
+   * corresponding to the mode given */
+  switch (mode)
+    {
+    case XNEE_GRAB_STOP:
+      xd->grab_keys->stop_key=km.key;
+      xd->grab_keys->stop_mod=km.modifier;
+      xnee_verbose((xd, "----  xnee_grab_key STOP mode\n"));
+      break;
+    case XNEE_GRAB_PAUSE:
+      xd->grab_keys->pause_key=km.key;
+      xd->grab_keys->pause_mod=km.modifier;
+      xnee_verbose((xd, "----  xnee_grab_key PAUSE mode\n"));
+      break;
+    case XNEE_GRAB_RESUME:
+      xd->grab_keys->resume_key=km.key;
+      xd->grab_keys->resume_mod=km.modifier;
+      xnee_verbose((xd, "----  xnee_grab_key RESUME mode\n"));
+      break;
+    default:
+      xnee_print_error ("Unknown grab mode\n");
+      return XNEE_UNKNOWN_GRAB_MODE;
+    }
 
+  /* make sure we have a display to grab on*/
   if (xd->grab==NULL)
     {
       xd->grab = XOpenDisplay (NULL);
+      if (xd->grab==NULL)
+	{
+	  xnee_verbose((xd, "could not open display for grab...\n"));
+	  return XNEE_NOT_OPEN_DISPLAY;
+	}
     }
-  if (xd->grab!=NULL)
-    {
-      screen = DefaultScreen (xd->grab);
-      window = RootWindow(xd->grab, screen );
-      xnee_verbose((xd, "window   %d\n", window));
-      xnee_verbose((xd, "screen   %d\n", screen));
-      xnee_verbose((xd, "data     %d\n", xd->grab));
-      xnee_verbose((xd, "stop key %d\n", xd->stop_key));
-      xnee_verbose((xd, "stop mod %d\n", xd->stop_mod));
-      ret = XGrabKey (xd->grab,  
-		      xd->stop_key,            
-		      xd->stop_mod,
-		      window,       
-		      True,  
-		      GrabModeAsync,
-		      GrabModeAsync );
-      xnee_verbose((xd, "grab OK\n"));
-    }
-  else
-    {
-      xnee_verbose((xd, "could not open display for grab...\n"));
-      return XNEE_NOT_OPEN_DISPLAY;
-    }
+  
+  /* grab key + modifier */
   screen = DefaultScreen (xd->grab);
   window = RootWindow    (xd->grab, screen );
   xnee_verbose((xd, "window   %d\n", window));
   xnee_verbose((xd, "screen   %d\n", screen));
   xnee_verbose((xd, "data     %d\n", xd->grab));
-  xnee_verbose((xd, "stop key %d\n", xd->stop_key));
-  xnee_verbose((xd, "stop mod %d\n", xd->stop_mod));
-  ret = XGrabKey (xd->grab,  
-		  xd->stop_key,            
-		  xd->stop_mod,
-		  window,       
-		  False,  
-		  GrabModeSync,
-		  GrabModeSync );
-  xnee_verbose((xd, "grab OK\n"));
+  xnee_verbose((xd, "stop key %d\n", km.key));
+  xnee_verbose((xd, "stop mod %d\n", km.modifier));
+  XGrabKey (xd->grab,  
+	    km.key,            
+	    km.modifier,
+	    window,       
+	    False,  
+	    GrabModeSync,
+	    GrabModeSync );
   xnee_verbose((xd, "<---- xnee_grab_stop_key\n"));
-  return ret;
+  return XNEE_OK;
 }
 
 
 
-/**************************************************************
- *                                                            *
- * xnee_ungrab_stop_key                                       *
- *                                                            *
- *                                                            *
- **************************************************************/
-int 
-xnee_ungrab_stop_key (xnee_data* xd)
+int
+xnee_get_grab_mode (xnee_data *xd, int key, int modifier)
 {
-  int ret;
+  
+
+
+  if ( (key==xd->grab_keys->stop_key) && 
+       (modifier==xd->grab_keys->stop_mod) )
+    {
+      xnee_verbose ((xd, "xnee_get_grab_mode: STOP \n"));
+      return XNEE_GRAB_STOP;
+    }
+  else if ( (key==xd->grab_keys->pause_key) && 
+	    (modifier==xd->grab_keys->pause_mod) )
+    {
+      xnee_verbose ((xd, "xnee_get_grab_mode: PAUSE \n"));
+      return XNEE_GRAB_PAUSE;
+    }
+  else if ( (key==xd->grab_keys->resume_key) && 
+	    (modifier==xd->grab_keys->resume_mod) )
+    {
+      xnee_verbose ((xd, "xnee_get_grab_mode: RESUME \n"));
+      return XNEE_GRAB_PAUSE;
+    }
+  else 
+   {
+ return XNEE_GRAB_UNKOWN;
+   }
+}
+
+
+int
+xnee_check_km(xnee_data *xd)
+{
+  XEvent ev;
+  if (xd->grab!=NULL)
+    {
+      /*
+       * Has the user pressed STOP (mod + key) 
+       */
+      XFlush (xd->grab);
+      XAllowEvents (xd->grab,
+		    AsyncKeyboard,
+		    CurrentTime);
+      XFlush (xd->grab);
+      
+      if ( !XCheckMaskEvent ( xd->grab, 0xffffffff , &ev) == False)
+	{
+	  XEvent my_event ;
+	  int    tmp_code;
+	  int    tmp_modifier;
+	  int    mode;
+	  XNextEvent (xd->grab, &my_event);
+	  if (my_event.xkey.send_event==1) 
+	    { 
+	      xnee_verbose ((xd , "send_event==true\n")); 
+	    }
+	  tmp_modifier=my_event.xkey.state;
+	  tmp_code=my_event.xkey.keycode;
+	  xnee_verbose ((xd, "key      = %d\n", tmp_modifier));
+	  xnee_verbose ((xd, "modifier = %d\n", tmp_code));
+	  mode=xnee_get_grab_mode (xd, tmp_code, tmp_modifier);
+	  xnee_verbose ((xd, "mode     = %d\n", mode));
+	  
+	  
+	  xnee_verbose((xd, "\n\n\tUSER PUSHED MOD + KEY ... leaving\n\n\n"));
+	  /* Since we've got called we have data .. we don't care about it. 
+	     Only freeeing the mem */
+	  
+	  return XNEE_GRAB_DATA;
+	}
+      else
+	{
+	  return XNEE_NO_GRAB_DATA;
+	}
+      
+    }
+  else
+    {
+      return XNEE_NO_GRAB_DATA;
+    }
+  return XNEE_NO_GRAB_DATA;
+}
+
+
+
+int 
+xnee_ungrab_key (xnee_data* xd, int mode)
+{
+  int key      = 0 ;
+  int modifier = 0 ;
   int window;
   int screen;
-  
-  xnee_verbose((xd, "---> xnee_ungrab_stop_key\n"));
+    
+  switch (mode)
+    {
+    case XNEE_GRAB_STOP:
+      key      = xd->grab_keys->stop_key;
+      modifier = xd->grab_keys->stop_mod;
+      xd->grab_keys->stop_key=0;
+      xd->grab_keys->stop_mod=0;
+      xnee_verbose((xd, "----  xnee_grab_key STOP mode\n"));
+      break;
+    case XNEE_GRAB_PAUSE:
+      key      = xd->grab_keys->pause_key;
+      modifier = xd->grab_keys->pause_mod;
+      xd->grab_keys->pause_key=0;
+      xd->grab_keys->pause_mod=0;
+      xnee_verbose((xd, "----  xnee_grab_key PAUSE mode\n"));
+      break;
+    case XNEE_GRAB_RESUME:
+      key      = xd->grab_keys->resume_key;
+      modifier = xd->grab_keys->resume_mod;
+      xd->grab_keys->resume_key=0;
+      xd->grab_keys->resume_mod=0;
+      xnee_verbose((xd, "----  xnee_grab_key RESUME mode\n"));
+      break;
+    default:
+      xnee_print_error ("Unknown grab mode\n");
+      return XNEE_UNKNOWN_GRAB_MODE;
+    }
+
   if ( xd->grab != 0 )
     {
       screen = DefaultScreen (xd->grab);
@@ -789,15 +942,31 @@ xnee_ungrab_stop_key (xnee_data* xd)
       xnee_verbose((xd, "window   %d\n", window));
       xnee_verbose((xd, "screen   %d\n", screen));
       xnee_verbose((xd, "data     %d\n", xd->grab));
-      xnee_verbose((xd, "stop key %d\n", xd->stop_key));
-      xnee_verbose((xd, "stop mod %d\n", xd->stop_mod));
-      ret = XUngrabKey (xd->grab,  
-			xd->stop_key,            
-			xd->stop_mod,
-			window);
+      xnee_verbose((xd, "stop key %d\n", key));
+      xnee_verbose((xd, "stop mod %d\n", modifier));
+      XUngrabKey (xd->grab,  
+		  key,            
+		  modifier,
+		  window);
     }
+}
+
+/**************************************************************
+ *                                                            *
+ * xnee_ungrab_keys                                           *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+int 
+xnee_ungrab_keys (xnee_data* xd)
+{
+  xnee_verbose((xd, "---> xnee_ungrab_stop_key\n"));
+  xnee_ungrab_key ( xd, XNEE_GRAB_STOP);
+  xnee_ungrab_key ( xd, XNEE_GRAB_PAUSE);
+  xnee_ungrab_key ( xd, XNEE_GRAB_RESUME);
+  
   xnee_verbose((xd, "<--- xnee_ungrab_stop_key\n"));
-  /* TODO : RETURN THE RETURN CODE FROM XUNGRABKEY CALL */
+  return XNEE_OK;
 }
 
 
@@ -811,13 +980,7 @@ int
 xnee_stop_session( xnee_data* xd)
 {
   xnee_verbose((xd, " ---> xnee_stop_session\n" ));
-  if ( xd->grab != 0 )
-    {
-      xd->cont=False ;
-      xd->xnee_info->loops_left=0 ;
-      xnee_ungrab_stop_key(xd);
-      XCloseDisplay (xd->grab);
-    }
+  xnee_ungrab_keys(xd);
   xnee_verbose((xd, " <--- xnee_stop_session\n" ));
   return (0);
 }
@@ -847,6 +1010,48 @@ signal_handler(int sig)
 
 
 
+/**************************************************************
+ *                                                            *
+ * xnee_new_grab_keys                                         *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+xnee_grab_keys  *
+xnee_new_grab_keys()
+{
+
+  xnee_grab_keys* xgk = 
+    (xnee_grab_keys*) malloc (sizeof (xnee_grab_keys));
+  /*  memset (xgk, 0, sizeof (xnee_grab_keys)); */
+
+  if (xgk==NULL)
+    return NULL;
+  xgk->stop_key     = 0 ;
+  xgk->stop_mod     = 0 ;
+
+  xgk->pause_key    = 0 ;
+  xgk->pause_mod    = 0 ;
+
+  xgk->resume_key   = 0 ;
+  xgk->resume_mod   = 0 ;
+
+
+  return xgk;
+}
+
+
+/**************************************************************
+ *                                                            *
+ * xnee_free_grab_keys                                        *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+int
+xnee_free_grab_keys(xnee_data *xd)
+{
+  free (xd->grab_keys);
+}
+
 
 
 /**************************************************************
@@ -902,7 +1107,7 @@ xnee_free_recordext_setup(xnee_data* xd)
   free (xd->record_setup->rState);
   free (xd->record_setup);
   XNEE_DEBUG ( (stderr ," <--xnee_free_xnee_data()\n"  ));
-  return 0;
+  return XNEE_OK;
 }
 
 
@@ -932,11 +1137,12 @@ xnee_new_xnee_data()
   memset (xd->replay_setup, 0, sizeof(xnee_testext_setup));
 
   xd->record_setup  = xnee_new_recordext_setup(); 
+
+  xd->grab_keys     = xnee_new_grab_keys();
   return xd;
 }
 
 
-
 /**************************************************************
  *                                                            *
  * xnee_free_xnee_data                                        *
@@ -947,25 +1153,14 @@ int
 xnee_free_xnee_data(xnee_data* xd)
 {
 
-  XNEE_DEBUG ( (stderr ," -->xnee_free_xnee_data()\n"  ));
-
-
-  XNEE_DEBUG ( (stderr, "    Freeing record_setup\n"));
   xnee_free_recordext_setup ( xd);
-
-  free ( xd->xnee_info );
-  XNEE_DEBUG ( (stderr, "  Freeing replay_setup\n"));
-  free ( xd->replay_setup );
-  
+  xnee_free_grab_keys(xd);
+  free (xd->xnee_info);
+  free (xd->replay_setup );
   free (xd->buf_sem);
-
-  XNEE_DEBUG ( (stderr, "Freeing xd\n"));
   free (xd);
-  XNEE_DEBUG (  (stderr, "  xnee_data memory freed\n"));
 
-  XNEE_DEBUG ( (stderr ," <--xnee_free_recordext_data()\n"  ));
-
-  return 0;
+  return XNEE_OK;
 }
 
 
@@ -1230,10 +1425,9 @@ int
 xnee_use_plugin(xnee_data *xd, char *pl_name)
 {
   int ret=0;
-  xnee_verbose ((xd, "Using plugin file: %s\n", pl_name));
-  
   char lib_name[100];
 
+  xnee_verbose ((xd, "Using plugin file: %s\n", pl_name));
   strcpy (lib_name, "lib");
   strcat (lib_name, pl_name);
   
@@ -1282,7 +1476,7 @@ xnee_use_plugin(xnee_data *xd, char *pl_name)
       */
     }
   
-   return 0;
+   return XNEE_OK;
 }
 
 
@@ -1300,7 +1494,7 @@ xnee_add_resource_syntax(xnee_data *xd, char *tmp)
   int ret=1;  
   char *range;
   int len=0;
-
+  
   len=strlen(tmp);
   
   if ( tmp == NULL)
@@ -1314,119 +1508,128 @@ xnee_add_resource_syntax(xnee_data *xd, char *tmp)
       return -1;
     }
   xnee_verbose((xd, "   adding (2) \"%s\"  \n", tmp));
-
+  
   if (!strncmp("#",tmp,1))  /* # META data */
     {
       ; /* for now ..... do nothing */
     }
-  else if (!strncmp("display",tmp,7))  /* # META data */
+  else if (!strncmp(XNEE_DISPLAY,tmp,strlen(XNEE_DISPLAY)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
-      /* MEMORY LEAK ....... FIX ME  */
-      xd->display = (char *)strdup (range);
+      xnee_set_display_name (xd, range) ;
     }
-  else if (!strncmp(XNEE_FIRST_LAST,tmp,strlen(XNEE_FIRST_LAST)))  /* # META data */
+  else if (!strncmp(XNEE_FIRST_LAST,tmp,strlen(XNEE_FIRST_LAST)))
     {
       xd->xnee_info->first_last = True;
     }
-  else if (!strncmp(XNEE_NO_SYNC,tmp,strlen(XNEE_NO_SYNC)))  /* # META data */
+  else if (!strncmp(XNEE_NO_SYNC,tmp,strlen(XNEE_NO_SYNC)))
     {
       xd->sync = False;
     }
-  else if (!strncmp(XNEE_FORCE_REPLAY,tmp,strlen(XNEE_FORCE_REPLAY)))  /* # META data */
+  else if (!strncmp(XNEE_FORCE_REPLAY,tmp,strlen(XNEE_FORCE_REPLAY)))
     {
       xd->force_replay = True;
     }
-  else if (!strncmp(XNEE_ALL_CLIENTS,tmp,strlen(XNEE_ALL_CLIENTS)))  /* # META data */
+  else if (!strncmp(XNEE_ALL_CLIENTS,tmp,strlen(XNEE_ALL_CLIENTS)))
     {
       xd->all_clients = True;
     }
-  else if (!strncmp("verbose",tmp,7))  /* # META data */
+  else if (!strncmp(XNEE_VERBOSE,tmp,strlen(XNEE_VERBOSE)))  
     {
       xd->verbose = True;
     }
-  else if (!strncmp("buffer-verbose",tmp,7))  /* # META data */
+  else if (!strncmp(XNEE_BUFFER_VERBOSE,tmp,strlen(XNEE_BUFFER_VERBOSE)))  
     {
       xd->buf_verbose = True;
     }
-  else if (!strncmp("loops",tmp,5))  /* # META data */
+  else if (!strncmp(XNEE_LOOPS,tmp,strlen(XNEE_LOOPS))) 
     {
       range=strstr (tmp, ":");
       range += 1 ;
-      /*	  range[strlen(range)-1]='\0';*/
-      
       xd->xnee_info->loops_left = atoi(range);
     }
-  else if (!strncmp(XNEE_STOP_KEY,tmp,strlen(XNEE_STOP_KEY)))  /* # META data */
+  else if (!strncmp(XNEE_STOP_KEY,tmp,strlen(XNEE_STOP_KEY)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
-      xnee_verbose((xd, "==================== Stop key :\"%s\"\n", range ));
-      xnee_add_stop_key (xd, range);
-      xnee_verbose((xd, "==================== Stop key :\"%s\"\n", range ));
+      xnee_verbose((xd, "Stop key :\"%s\"\n", range ));
+      xnee_grab_key (xd, XNEE_GRAB_STOP, range);
+      xnee_verbose((xd, "Stop key :\"%s\"\n", range ));
     }
-  else if (!strncmp(XNEE_OUT_FILE,tmp,3))  /* # META data */
+  else if (!strncmp(XNEE_PAUSE_KEY,tmp,strlen(XNEE_PAUSE_KEY)))
+    {
+      range=strstr (tmp, ":");
+      range += 1 ;
+      xnee_verbose((xd, "Pause key :\"%s\"\n", range ));
+      xnee_grab_key (xd, XNEE_GRAB_PAUSE, range);
+      xnee_verbose((xd, "Pause key :\"%s\"\n", range ));
+    }
+  else if (!strncmp(XNEE_RESUME_KEY,tmp,strlen(XNEE_RESUME_KEY)))
+    {
+      range=strstr (tmp, ":");
+      range += 1 ;
+      xnee_verbose((xd, "Resume key :\"%s\"\n", range ));
+      xnee_grab_key (xd, XNEE_GRAB_RESUME, range);
+      xnee_verbose((xd, "Resume key :\"%s\"\n", range ));
+    }
+  else if (!strncmp(XNEE_KEYBOARD,tmp,strlen(XNEE_KEYBOARD))) 
+    {	 
+      xnee_parse_range (xd, XNEE_DEVICE_EVENT, 
+			"KeyPress-KeyRelease");
+      
+    }
+  else if (!strncmp(XNEE_MOUSE,tmp,strlen(XNEE_MOUSE)))
+    {
+      xnee_parse_range (xd, XNEE_DEVICE_EVENT, 
+			"ButtonPress-MotionNotify");
+    }
+  else if (!strncmp(XNEE_OUT_FILE,tmp,3)) 
     {
       xnee_verbose((xd, "file \n" ));
       range=strstr (tmp, ":");
       range += 1 ;
-      /*	  range[strlen(range)-1]='\0';*/
-      /* MEMORY LEAK ....... FIX ME  */
-      
       xnee_verbose((xd, "file \"%s\"\n", range));
-      xd->out_name = (char *)strdup (range);
-      xd->out = fopen (range,"w");
+      xnee_set_out_byname (xd, range);
     }
-  else if (!strncmp(XNEE_ERR_FILE,tmp,8))  /* # META data */
+  else if (!strncmp(XNEE_ERR_FILE,tmp,8))  
     {
       range=strstr (tmp, ":");
       range += 1 ;
-      /*	  range[strlen(range)]='\0';
-		  range[strlen(range)-1]='\0';*/
-      /* MEMORY LEAK ....... FIX ME  */
-      xd->err_name = (char *)strdup (range);
-      xd->err = fopen (range,"w");
+      xnee_set_err_byname (xd, range);
     }
-  else if (!strncmp("display",tmp,7))  /* # META data */
-    {
-      range=strstr (tmp, ":");
-      range += 1 ;
-      /*	  range[strlen(range)-1]='\0';*/
-      /* MEMORY LEAK ....... FIX ME  */
-      xd->display = (char *)strdup (range);
-    }
-  else if (!strncmp("distribute",tmp,10))  /* # META data */
+  else if (!strncmp(XNEE_DISTRIBUTE,tmp,strlen(XNEE_DISTRIBUTE)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_add_display_list ( xd, range);
     }
-  else if (!strncmp(XNEE_DEVICE_EVENT_STR,tmp,strlen(XNEE_DEVICE_EVENT_STR)))  /* # META data */
+  else if (!strncmp(XNEE_DEVICE_EVENT_STR,tmp,strlen(XNEE_DEVICE_EVENT_STR)))
     {
       range=strstr (tmp, ":");
       range ++ ;
       xnee_parse_range ( xd, XNEE_DEVICE_EVENT, range);
     }
-  else if (!strncmp(XNEE_DELIVERED_EVENT_STR,tmp,strlen(XNEE_DELIVERED_EVENT_STR)))  /* # META data */
+  else if (!strncmp(XNEE_DELIVERED_EVENT_STR,
+		    tmp,strlen(XNEE_DELIVERED_EVENT_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_DELIVERED_EVENT, range);
     }
-  else if (!strncmp(XNEE_ERROR_STR,tmp,strlen(XNEE_ERROR_STR)))  /* # META data */
+  else if (!strncmp(XNEE_ERROR_STR,tmp,strlen(XNEE_ERROR_STR))) 
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_ERROR, range);
     }
-  else if (!strncmp(XNEE_REQUEST_STR,tmp,strlen(XNEE_REQUEST_STR)))  /* # META data */
+  else if (!strncmp(XNEE_REQUEST_STR,tmp,strlen(XNEE_REQUEST_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_REQUEST, range);
     }
-  else if (!strncmp(XNEE_REPLY_STR,tmp,strlen(XNEE_REPLY_STR)))  /* # META data */
+  else if (!strncmp(XNEE_REPLY_STR,tmp,strlen(XNEE_REPLY_STR)))  
     {
       range=strstr (tmp, ":");
       range += 1 ;
@@ -1464,84 +1667,90 @@ xnee_add_resource_syntax(xnee_data *xd, char *tmp)
    * ... UNTIL ???  
    */
   /* START OF OBSOLETE PARSING */
-  else if (!strncmp("first_last",tmp,7))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_FIRST_LAST,tmp,strlen(XNEE_OBSOLETE_FIRST_LAST)))
     {
       xd->xnee_info->first_last = True;
     }
-  else if (!strncmp("no_sync",tmp,7))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_NO_SYNC,tmp,strlen(XNEE_OBSOLETE_NO_SYNC)))  
     {
       xd->sync = False;
     }
-  else if (!strncmp("all_clients",tmp,11))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_ALL_CLIENTS,tmp,strlen(XNEE_OBSOLETE_ALL_CLIENTS)))
     {
       xd->all_clients = True;
     }
-  else if (!strncmp("stop_key",tmp,8))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_STOP_KEY,tmp,strlen(XNEE_OBSOLETE_STOP_KEY)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_verbose((xd, "==================== Stop key :\"%s\"\n", range ));
-      xnee_add_stop_key (xd, range);
+      xnee_grab_key (xd, XNEE_GRAB_STOP, range);
       xnee_verbose((xd, "==================== Stop key :\"%s\"\n", range ));
     }
-  else if (!strncmp("err_file",tmp,8))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_ERROR_STR,tmp,strlen(XNEE_OBSOLETE_ERROR_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       if ( xnee_use_plugin(xd, range) != 0 )
 	{
-	  xnee_print_error ("Unable to open plugin file (%s)\n", range);
+	xnee_print_error ("Unable to open plugin file (%s)\n", range);
 	}
     }
-  else if (!strncmp(XNEE_OBSOLETE_DEVICE_EVENT_STR,tmp,strlen(XNEE_OBSOLETE_DEVICE_EVENT_STR)))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_DEVICE_EVENT_STR,tmp,
+		    strlen(XNEE_OBSOLETE_DEVICE_EVENT_STR)))
     {
       range=strstr (tmp, ":");
       range ++ ;
       xnee_parse_range ( xd, XNEE_DEVICE_EVENT, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_DELIVERED_EVENT_STR,tmp,strlen(XNEE_OBSOLETE_DELIVERED_EVENT_STR)))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_DELIVERED_EVENT_STR,tmp,
+		    strlen(XNEE_OBSOLETE_DELIVERED_EVENT_STR))) 
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_DELIVERED_EVENT, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_ERROR_STR,tmp,strlen(XNEE_OBSOLETE_ERROR_STR)))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_ERROR_STR,tmp,strlen(XNEE_OBSOLETE_ERROR_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_ERROR, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_REQUEST_STR,tmp,strlen(XNEE_OBSOLETE_REQUEST_STR)))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_REQUEST_STR,tmp,strlen(XNEE_OBSOLETE_REQUEST_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_REQUEST, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_REPLY_STR,tmp,strlen(XNEE_OBSOLETE_REPLY_STR)))  /* # META data */
+  else if (!strncmp(XNEE_OBSOLETE_REPLY_STR,tmp,strlen(XNEE_OBSOLETE_REPLY_STR))) 
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_REPLY, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_EXT_REQ_MAJ_STR,tmp,strlen(XNEE_OBSOLETE_EXT_REQ_MAJ_STR)))
+  else if (!strncmp(XNEE_OBSOLETE_EXT_REQ_MAJ_STR,tmp,
+		    strlen(XNEE_OBSOLETE_EXT_REQ_MAJ_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_EXT_REQUEST_MAJOR, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_EXT_REQ_MIN_STR,tmp,strlen(XNEE_OBSOLETE_EXT_REQ_MIN_STR)))
+  else if (!strncmp(XNEE_OBSOLETE_EXT_REQ_MIN_STR,tmp,
+		    strlen(XNEE_OBSOLETE_EXT_REQ_MIN_STR)))
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_EXT_REQUEST_MINOR, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_EXT_REP_MAJ_STR,tmp,strlen(XNEE_OBSOLETE_EXT_REP_MAJ_STR))) 
+  else if (!strncmp(XNEE_OBSOLETE_EXT_REP_MAJ_STR,tmp,
+		    strlen(XNEE_OBSOLETE_EXT_REP_MAJ_STR))) 
     {
       range=strstr (tmp, ":");
       range += 1 ;
       xnee_parse_range ( xd, XNEE_EXT_REPLY_MAJOR, range);
     }
-  else if (!strncmp(XNEE_OBSOLETE_EXT_REP_MIN_STR,tmp,strlen(XNEE_OBSOLETE_EXT_REP_MIN_STR))) 
+  else if (!strncmp(XNEE_OBSOLETE_EXT_REP_MIN_STR,tmp,
+		    strlen(XNEE_OBSOLETE_EXT_REP_MIN_STR))) 
     {
       range=strstr (tmp, ":");
       range ++ ;
@@ -1581,6 +1790,7 @@ rem_blanks (char *array, int size)
 	  array[j]='\0';
 	}
     }
+  return XNEE_OK;
 }
 
 
@@ -1611,30 +1821,46 @@ rem_all_blanks (char *array, int size)
 	  array[j-zeros]='\0';
 	}
     }
+  return XNEE_OK;
 }
 
 
 
+
+
 /**************************************************************
  *                                                            *
- * xnee_add_stop_key                                          *
+ * xnee_get_km_tuple                                          *
  *                                                            *
  *                                                            *
  **************************************************************/
 int
-xnee_add_stop_key (xnee_data *xd, char *mod_and_key)
+xnee_get_km_tuple (xnee_data     *xd, 
+		   xnee_km_tuple *km, 
+		   char          *mod_and_key)
 {
   char key_buf[20];
   char mod_buf[20];
-  char *next;
-  char *tmp;
   int i=0;
   int idx=-1;
   int len=strlen(mod_and_key);
 
-  xnee_verbose((xd, " ---> xnee_add_stop_key \n"));
-  rem_all_blanks (mod_and_key, strlen(mod_and_key));
-  xnee_verbose((xd, " ---  xnee_add_stop_key stripped string:\"%s\"\n", mod_and_key));
+  xnee_verbose((xd, " ---> xnee_get_km_tuple_key %s \n", mod_and_key));
+  strcpy (mod_buf, mod_and_key);
+  rem_all_blanks (mod_buf, strlen(mod_and_key));
+
+
+  /* make sure we have a display to grab on */
+  if ( xd->grab == NULL)
+    {
+      xd->grab = XOpenDisplay (xd->display);
+      if ( xd->grab == NULL)
+	{
+	  xnee_print_error ("Could not open display for grabbing\n");
+	  return (XNEE_NOT_OPEN_DISPLAY);
+	}
+    }
+  
 
   for (i=0;i<len;i++)
     {
@@ -1658,21 +1884,16 @@ xnee_add_stop_key (xnee_data *xd, char *mod_and_key)
 	}
     }
 
-  xnee_verbose((xd, " ---  xnee_add_stop_key mod=\"%s\"   key=\"%s\"\n", mod_buf, key_buf));
-
-  if (xd->grab==NULL)
-    {
-      xd->grab = XOpenDisplay (NULL);
-    }
+  xnee_verbose((xd, " ---  xnee_get_km_tuple_key mod=\"%s\"   key=\"%s\"\n", mod_buf, key_buf));
 
   /* Is it a modifier in alias form (e.g Control, Shift....)
      or is it a number */
   i=get_modifier(xd, mod_buf);
   if ( i != -1 )
     {
-      xd->stop_mod=i;
+      km->modifier=i;
     }
-  else if ( sscanf ( mod_buf, "%d", &xd->stop_mod) < 0 )
+  else if ( sscanf ( mod_buf, "%d", km->modifier) < 0 )
     {
       xnee_print_error ("SYNTAX ERROR\n\tCouldn't convert "
 			" string \"%s\" to an int\n",mod_buf);
@@ -1682,7 +1903,7 @@ xnee_add_stop_key (xnee_data *xd, char *mod_and_key)
   
   /* Is it a Key in letter form (e.g a,b)
      or is it a number */
-  if ( sscanf ( key_buf, "%d", &xd->stop_key) <= 0 )
+  if ( sscanf ( key_buf, "%d", km->key) <= 0 )
     {
       if (strlen(key_buf)!=1)
 	{
@@ -1693,36 +1914,19 @@ xnee_add_stop_key (xnee_data *xd, char *mod_and_key)
       else
 	{
 	  /* is it a letter */
-	  xd->stop_key = XStringToKeysym (key_buf)  ; 
-	  if ( xd->stop_key == 0 )
+	  km->key = XStringToKeysym (key_buf)  ; 
+	  if ( km->key == 0 )
 	    {
 	      xnee_print_error ("SYNTAX ERROR\n\tCouldn't convert "
 				"string \"%s\" to an int\n",key_buf);
 	      return ( XNEE_SYNTAX_ERROR);
 	    }
-	  xd->grab = XOpenDisplay (xd->display);
-	  if ( xd->grab ==NULL)
-	    {
-	      xnee_print_error ("Could not open display \"%s\"\n", 
-				xd->display);
-	      return (XNEE_NOT_OPEN_DISPLAY);
-	    }
 	  /* OK, it's a integer... just use it! */
-	  xd->stop_key = XKeysymToKeycode (xd->grab, xd->stop_key);
+	  km->key = XKeysymToKeycode (xd->grab, km->key);
 	}
     } 
-  else
-    {
-      xd->grab = XOpenDisplay (xd->display);
-      if ( xd->grab ==NULL)
-	{
-	  xnee_print_error ("Could not open display \"%s\"\n", 
-			    xd->display);
-	  return (XNEE_NOT_OPEN_DISPLAY);
-	}
-    }
   xnee_verbose((xd, " <--- xnee_add_stop_key \n"));
-  return xnee_grab_stop_key(xd);
+  return XNEE_OK;
 }
 
 
@@ -1746,8 +1950,8 @@ xnee_add_resource(xnee_data *xd)
 
   while (read_more!=0)
     {
-      if ( fgets(tmp, 256, xd->sf) == NULL)
-	return 0;
+      if ( fgets(tmp, 256, xd->rc_file) == NULL)
+	return XNEE_OK;
       /*
        * Hey, I know we'll keep the char array....
        * as long as we need... next call aint deep 
@@ -1789,7 +1993,7 @@ xnee_get_record_config (xnee_data* xd, xnee_intercept_data * xindata)
       xnee_verbose ((xd, "tmp: >%s", tmp));
       if ( tmp == NULL) 
 	{
-	  return 0;
+	  return XNEE_OK;
 	}
       if (tmp[0] == '#' )  /* # META data */
 	{
@@ -1846,7 +2050,8 @@ xnee_check_inSync (xnee_data *xd )
  *                                                            *
  *                                                            *
  **************************************************************/
-long xnee_get_elapsed_time(xnee_data *xd, char type )
+long 
+xnee_get_elapsed_time(xnee_data *xd, char type )
 {
   static  long last_read_msecec = 0;  /* from last call of this f'n */
   static  long last_read_sec = 0;
@@ -1885,7 +2090,7 @@ long xnee_get_elapsed_time(xnee_data *xd, char type )
       time_offset_msec = (diff_sec * XNEE_USEC_PER_SEC) + diff_msec;
 
 
-      /* if first time through - save values and return 0 */
+      /* if first time through - save values and return XNEE_OK */
       if ( last_sec == 0  )
 	{
 	  last_read_msecec = cur_time.tv_usec;
@@ -1894,7 +2099,7 @@ long xnee_get_elapsed_time(xnee_data *xd, char type )
 	  first_read_sec = cur_time.tv_sec;
 /* 	  xnee_verbose ((xd, "1st elapsed time:%c _sec _msec: %d %d timeoffeset = 0\n",  */
 /* 		       type, cur_time.tv_sec, cur_time.tv_usec )); */
-	  return 0;
+	  return XNEE_OK;
 	}
 
       /* update last_read values with the new time */
@@ -1906,8 +2111,10 @@ long xnee_get_elapsed_time(xnee_data *xd, char type )
 /*       xnee_verbose ((xd, "elapsed time type:%c current time _sec _msec: %d %d time_offset_msec: %d\n",  */
 /* 		   type, cur_time.tv_sec, cur_time.tv_usec, time_offset_msec )); */
       return time_offset_msec;
-
     }
+
+  /* Shouldn't reach this point.... silent the compiler */
+  return 0;
 }
 
 /**************************************************************
@@ -1982,14 +2189,14 @@ xnee_calc_sleep_amount(xnee_data *xd, long last_diff, long first_diff, long reco
  **************************************************************/
 int
 xnee_set_callback (xnee_data *xd, 
-		   void (**dest) (XPointer , XRecordInterceptData *), 
+		   callback_ptrptr dest, 
 		   char *sym_name)
 {
   char *error;
-  void *saved;
+  callback_ptr saved;
   xnee_verbose ((xd, "\nTrying to set \"%s\" as callback\n", sym_name));
 
-  saved = dest;
+  saved = *dest;
 
   if (xd==NULL)
     {
@@ -1997,14 +2204,13 @@ xnee_set_callback (xnee_data *xd,
     }
   else
     {
-      *dest = (void*) xnee_dlsym(xd, 
-			 xd->plugin_handle,
-			 sym_name);
-      
-      if ((error = (char*)xnee_dlerror()) != NULL)  
+      *dest = (callback_ptr) xnee_dlsym(xd, 
+					 xd->plugin_handle,
+					 sym_name);
+      if ((error = (char*)xnee_dlerror(xd)) != NULL)  
 	{
 	  xnee_verbose ((xd, "Failed to set \"%s\" from plugin\n", sym_name));
-	  dest = saved ;
+	  *dest = saved ;
 	  fputs(error, stderr);
 	  return (XNEE_PLUGIN_FILE_ERROR);
 	}
@@ -2019,21 +2225,21 @@ xnee_set_callback (xnee_data *xd,
 
 /**************************************************************
  *                                                            *
- * xnee_set_callback                                          *
+ * xnee_set_synchronize                                       *
  *                                                            *
  *                                                            *
  **************************************************************/
 int
 xnee_set_synchronize (xnee_data *xd,
-		      void (**dest) (xnee_data *xd, int replayed_type, int replayed_nr), 
+		      synch_ptrptr dest, 
 		      char *sym_name)
 {
 
   char *error;
-  void *saved;
+  synch_ptr saved;
   xnee_verbose ((xd, "\nTrying to set \"%s\" as callback\n", sym_name));
 
-  saved = dest;
+  saved = *dest;
 
   if (xd==NULL)
     {
@@ -2041,14 +2247,14 @@ xnee_set_synchronize (xnee_data *xd,
     }
   else
     {
-      *dest = (void*) xnee_dlsym(xd, 
-				xd->plugin_handle, 
-				sym_name);
-
-      if ((error = (char*)xnee_dlerror()) != NULL)  
+      *dest = (synch_ptr) xnee_dlsym(xd, 
+				     xd->plugin_handle, 
+				     sym_name);
+      
+      if ((error = (char*)xnee_dlerror(xd)) != NULL)  
 	{
 	  xnee_verbose ((xd, "Failed to set \"%s\" from plugin\n", sym_name));
-	  dest = saved ;
+	  *dest = saved ;
 	  fputs(error, stderr);
 	  return (XNEE_PLUGIN_FILE_ERROR);
 	}
