@@ -39,10 +39,12 @@
  *                                                            *
  **************************************************************/
 void 
-xnee_record_dispatch2(XPointer xpoint_xnee_data,
-			  XRecordInterceptData *data )
+xnee_record_dispatch2 (XPointer xpoint_xnee_data,
+		       XRecordInterceptData *data )
 {
-  printf ("."); fflush (stdout);
+  xnee_data           *xd ;
+  xd = (xnee_data *) (xpoint_xnee_data) ;
+  xd->data_fp (xd->out_file, "."); fflush (stdout);
   XRecordFreeData(data);
 }
 
@@ -60,7 +62,7 @@ xnee_record_handle_event ( xnee_data *xd, XRecordInterceptData *xrecintd)
 {
   XRecordDatum *xrec_data  = (XRecordDatum *) (xrecintd->data) ;
   int           event_type = xrec_data->type ;
-  FILE *out = xd->out;
+  FILE *out = xd->out_file;
   
 
   XNEE_DEBUG ( (stderr ," -->xnee_record_handle_event()  \n"  ));
@@ -209,7 +211,6 @@ xnee_record_dispatch(XPointer xpoint_xnee_data,
   XRecordDatum        *xrec_data  ;
   xnee_data           *xd ;
   int                  data_type = 0;  
-  XEvent ev;
 
   XNEE_DEBUG ( (stderr ," --> xnee_record_dispatch()  \n"  ));
 
@@ -225,46 +226,18 @@ xnee_record_dispatch(XPointer xpoint_xnee_data,
   xd = (xnee_data *) (xpoint_xnee_data) ;
   xrec_data = (XRecordDatum *) (data->data) ;
 
-  /*
-   *
-   * This flag is used to prevent recording of KeyRelease when starting
-   * the app. Solve the problem ASAP!!!
-   *
-   */
-  if (xd->first_replayed_event==1) 
+  /* make sure we haven't been interupted by a
+     a key + modifier combination */
+  if (xnee_check_km(xd)==XNEE_GRAB_DATA)
     {
       XRecordFreeData(data);
-      xd->first_replayed_event=0;
-      XNEE_DEBUG ( (stderr ," <-- xnee_record_dispatch()  \n"  ));
-      return ; 
-  }
- 
-
-  if (xd->grab!=NULL)
-    {
-      /*
-       * Has the user pressed STOP (mod + key) 
-       */
-      XFlush (xd->grab);
-      XAllowEvents (xd->grab,
-		    AsyncKeyboard,
-		    CurrentTime);
-      XFlush (xd->grab);
-  
-      if ( !XCheckMaskEvent ( xd->grab, 0xffffffff , &ev) == False)
-	{
-	  xnee_verbose((xd, "\n\n\tUSER PUSHED MOD + KEY ... leaving\n\n\n"));
-	  /* Since we've got called we have data .. we don't care about it. 
-	     Only freeeing the mem */
-	  XRecordFreeData(data);
-	  XRecordDisableContext(xd->control, 
-				xd->record_setup->rContext);
-	  xnee_stop_session(xd);
-	  return ;
-	}
+      XRecordDisableContext(xd->control, 
+			    xd->record_setup->rContext);
+      xnee_stop_session(xd);
+      return ;
     }
 
-
+  
   if ( xd->xnee_info->loops_left == 0 ) 
     {
       XRecordFreeData(data);
@@ -755,7 +728,15 @@ xnee_parse_range (xnee_data *xd,int type, char *range)
   int len;
   int ret=0;
   int range_len=strlen(range);
-  xnee_verbose((xd, "--> parse_range (%d, %d, %s)\n", (int) xd, type, range));
+
+  printf ("error file=%s\n", xd->err_name);
+  printf ("error file=%d\n", xd->err_file);
+
+  xnee_verbose ((xd,"int arg=%d\n", xd));
+  xnee_verbose ((xd, "nt arg=%d\n", type));
+  xnee_verbose ((xd, "string arg=%s\n", range));
+
+  xnee_verbose((xd, "--> parse_range (%u, %d, %s)\n", (int) xd, type, range));
 
   while ( 1 ) {
     next=strspn (range, "0123456789abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ-");
@@ -790,23 +771,23 @@ int
 xnee_setup_recordext (xnee_data *xd)
 {
   int ret=XNEE_OK;
-  xnee_verbose((xd, "xnee_setup_recordext 1   max=%d\n",xnee_get_max_range(xd)));
 
+  xnee_verbose((xd, " ---> xnee_setup_recordext\n"));
   xd->record_setup->data_flags = XRecordFromServerTime 
     | XRecordFromClientTime  
     | XRecordFromClientSequence; 
-  xnee_verbose((xd, "xnee_setup_recordext 2   max=%d\n",xnee_get_max_range(xd)));
-  xnee_verbose((xd, "xnee_setup_recordext 3   max=%d\n",xnee_get_max_range(xd)));
-  xnee_verbose((xd, "xnee_setup_recordext 4   max=%d\n",xnee_get_max_range(xd)));
   ret = xnee_get_max_range(xd) ;
-  xnee_verbose((xd, "xnee_setup_recordext 4   max=%d\n",ret));
   if (ret==0)
     {
-      xnee_verbose((xd, " Nothing to record .... "));
+      xnee_verbose((xd, " --- xnee_setup_recordext Nothing to record .... "));
       ret = XNEE_NO_PROT_CHOOSEN;
-    } ;
-  xnee_verbose((xd, "xnee_setup_recordext 4\n"));
+    } 
+  else
+    {
+      ret = XNEE_OK;
+    }
 
+  xnee_verbose((xd, " <--- xnee_setup_recordext %d\n", ret));
   return (ret);
 }
 
@@ -935,6 +916,14 @@ xnee_record_loop(xnee_data *xd)
 {
   xnee_verbose((xd, " ---> xnee_record_loop()\n"));
   
+  /* 
+   * In case the key pressed to invoke Xnee is not released
+   * we wait 1/2 of a second and hopefully it is. If not
+   * the user is holding it pressed for "TOO" long.
+   */
+  usleep ( XNEE_DELAY_RECORDING );
+
+
   XRecordEnableContext(xd->data, 
 		       xd->record_setup->rContext, 
 		       xd->rec_callback, 
@@ -950,6 +939,40 @@ xnee_record_loop(xnee_data *xd)
 }
 
 
+/**************************************************************
+ *                                                            *
+ * xnee_record_loop                                           *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+int 
+xnee_record_async(xnee_data *xd)
+{
+  xnee_verbose((xd, " ---> xnee_record_async()\n"));
+  
+  /* 
+   * In case the key pressed to invoke Xnee is not released
+   * we wait 1/2 of a second and hopefully it is. If not
+   * the user is holding it pressed for "TOO" long.
+   */
+  usleep ( XNEE_DELAY_RECORDING );
+
+
+  XRecordEnableContextAsync(xd->data, 
+		       xd->record_setup->rContext, 
+		       xd->rec_callback, 
+		       (XPointer) (xd) /* closure passed to Dispatch */);
+      
+  xnee_verbose((xd, " <--- xnee_record_async()\n"));
+  /*  while (1) 
+    {
+      XRecordProcessReplies (xd->data); 
+    }
+  */
+  return (0);
+}
+
+
 
 
 
